@@ -37,6 +37,10 @@
 #include <unistd.h>
 #endif
 
+#if USE_MINGW_ANSI_STDIO
+#define vsnprintf __mingw_vsnprintf
+#endif
+
 /* constants
  * =========
  */
@@ -682,7 +686,7 @@ std::string Command::filter(const std::string &command_line, const std::string s
     while (1)
     {
       char buffer[BUFSIZ];
-      size_t n = read(pipe_fds[0], buffer, sizeof buffer);
+      ssize_t n = read(pipe_fds[0], buffer, sizeof buffer);
       if (n < 0)
         throw_posix_error("read");
       else if (n == 0)
@@ -1257,6 +1261,16 @@ std::string string_vprintf(const char *message, va_list args)
   return static_cast<char*>(buffer);
 }
 
+std::string string_printf(const char *message, ...)
+{
+  va_list args;
+  va_start(args, message);
+  std::string result = string_vprintf(message, args);
+  va_end(args);
+  return result;
+}
+
+
 void prevent_pop_out(void)
 {
 #if WIN32
@@ -1279,13 +1293,4 @@ void prevent_pop_out(void)
 #endif
 }
 
-std::string string_printf(const char *message, ...)
-{
-  va_list args;
-  va_start(args, message);
-  std::string result = string_vprintf(message, args);
-  va_end(args);
-  return result;
-}
-
 // vim:ts=2 sw=2 et
